/*-------------------------------------------------------------------------+
| Copyright 2021 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.ext.reuse.ui.commands;

import static java.lang.String.join;
import static java.util.Collections.emptyList;
import static org.conqat.ide.commons.ui.dialog.MessageUtils.confirm;
import static org.conqat.ide.commons.ui.dialog.MessageUtils.showInfo;
import static org.eclipse.jface.window.Window.OK;
import static org.fortiss.tooling.ext.reuse.ui.utils.ReuseLibraryUIUtils.checkForElementInLibrary;
import static org.fortiss.tooling.ext.reuse.ui.utils.ReuseLibraryUIUtils.manageExternalReferencesOfReuseElement;
import static org.fortiss.tooling.ext.reuse.ui.utils.ReuseLibraryUIUtils.selectCorrectReuseSpec;
import static org.fortiss.tooling.ext.reuse.ui.utils.ReuseLibraryUIUtils.warnOfMissingLib;
import static org.fortiss.tooling.ext.reuse.utils.ReuseLibraryUtils.fixIds;
import static org.fortiss.tooling.ext.reuse.utils.ReuseLibraryUtils.getLibraryIDName;
import static org.fortiss.tooling.ext.reuse.utils.ReuseLibraryUtils.getLocalReuseLibraryByID;
import static org.fortiss.tooling.ext.reuse.utils.ReuseLibraryUtils.getReuseElementName;
import static org.fortiss.tooling.ext.reuse.utils.ReuseLibraryUtils.isCorrectReuseElement;
import static org.fortiss.tooling.ext.reuse.utils.ReuseLibraryUtils.replaceElementInProject;
import static org.fortiss.tooling.kernel.ui.service.IContextMenuService.TOP_MOST_MENU_SECTION_ID;
import static org.fortiss.tooling.kernel.utils.EcoreUtils.getParentsWithType;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.ActionContributionItem;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.swt.widgets.Display;
import org.fortiss.tooling.ext.reuse.model.ReuseElementSpec;
import org.fortiss.tooling.ext.reuse.model.ReuseLibrary;
import org.fortiss.tooling.ext.reuse.ui.ToolingReuseUIActivator;
import org.fortiss.tooling.kernel.model.INamedCommentedElement;
import org.fortiss.tooling.kernel.service.ICommandStackService;
import org.fortiss.tooling.kernel.ui.extension.IContextMenuContributor;
import org.fortiss.tooling.kernel.ui.extension.data.ContextMenuContextProvider;

/**
 * Context menu contributions to include updating an element in an AF3 project
 * from a {@link ReuseLibrary}.
 *
 * @author bergemann
 */
public class UpdateFromLibContextMenu implements IContextMenuContributor {

	/** The title of the dialog shell(s). */
	private final String shellTitle = "Update Element From Reuse Library";

	/** {@inheritDoc} */
	@Override
	public List<IContributionItem> getContributedItems(EObject selection, ContextMenuContextProvider contextProvider) {
		if (selection != null && canBeUpdated(selection)) {
			UpdateFromLibAction action = new UpdateFromLibAction(selection);
			List<IContributionItem> contributionItems = new ArrayList<IContributionItem>();
			contributionItems.add(new ActionContributionItem(action));
			return contributionItems;
		}
		return emptyList();
	}

	/** {@inheritDoc} */
	@Override
	public String getMenuSectionID() {
		return TOP_MOST_MENU_SECTION_ID;
	}

	/**
	 * Checks whether the given element has a link to a {@link ReuseLibrary} and can
	 * be updated from a {@link ReuseLibrary}.
	 */
	protected boolean canBeUpdated(EObject element) {
		return isCorrectReuseElement(element);
	}

	/**
	 * Update the given element from its source in the specified
	 * {@link ReuseLibrary}. The {@link ReuseElementSpec} is the one that connects
	 * reuse element and {@link ReuseLibrary}.
	 */
	protected void updateElemFromLib(EObject elementToBeUpdated, ReuseLibrary library, ReuseElementSpec referencingSpec,
			String newName) {
		if (library == null || elementToBeUpdated == null || referencingSpec == null || newName == null) {
			return;
		}

		// find original element inside the library
		EObject originalLibraryElement = checkForElementInLibrary(library, referencingSpec, shellTitle);
		if (originalLibraryElement == null) {
			return;
		}

		EObject updatedElement = replaceElementInProject(elementToBeUpdated, originalLibraryElement, newName);
		if (updatedElement == null) {
			String infoTitle = shellTitle + " - Replacement Failed";
			String infoMessage = "The replacement of '" + getReuseElementName(elementToBeUpdated) + "' with '"
					+ getReuseElementName(originalLibraryElement) + "' has failed.";
			showInfo(infoTitle, infoMessage);
			return;
		}

		fixIds(updatedElement);

		String currentActionType = "updateFromLibrary";
		manageExternalReferencesOfReuseElement(originalLibraryElement, currentActionType, shellTitle);
	}

	/** The action "update element from {@link ReuseLibrary}". */
	private class UpdateFromLibAction extends Action {

		/** The selected element. */
		private EObject selectedElement;

		/** Constructor. Takes the selected element as input. */
		UpdateFromLibAction(EObject element) {
			super("Update From Reuse Library",
					ToolingReuseUIActivator.getImageDescriptor("icons/update_from_library.png"));
			selectedElement = element;
		}

		/** {@inheritDoc} */
		@Override
		public void run() {
			ReuseElementSpec correctLibrarySpec = selectCorrectReuseSpec(selectedElement, shellTitle);
			if (correctLibrarySpec == null) {
				return;
			}
			ReuseLibrary correctLibrary = getLocalReuseLibraryByID(correctLibrarySpec.getSourceLibUUID());

			if (correctLibrary == null) {
				warnOfMissingLib(correctLibrarySpec, shellTitle);
				return;
			}

			String currentNameInProject = getReuseElementName(selectedElement);
			String currentProjectName = "not found";
			EList<INamedCommentedElement> parentList = getParentsWithType(selectedElement,
					INamedCommentedElement.class);
			if (parentList != null && !parentList.isEmpty()) {
				// safe due to check above and last element is the highest parent aka root
				INamedCommentedElement root = parentList.get(parentList.size() - 1);
				currentProjectName = root.getName();
			}

			// find original element inside the library
			EObject originalLibraryElement = checkForElementInLibrary(correctLibrary, correctLibrarySpec, shellTitle);
			if (originalLibraryElement == null) {
				return;
			}

			// confirm the update
			String currentNameInLibrary = getReuseElementName(originalLibraryElement);
			List<String> textLines = new ArrayList<String>();
			textLines.add("Confirm the following update (from source to target):");
			textLines.add("\n- Update source: corresponding element in library");
			textLines.add("    > element name: " + currentNameInLibrary);
			textLines.add("    > in library: " + getLibraryIDName(correctLibrary));
			textLines.add("\n- Update target: element in current project");
			textLines.add("    > element name: " + currentNameInProject);
			textLines.add("    > in project: " + currentProjectName);
			boolean confirmed = confirm(shellTitle, join("\n", textLines));

			if (!confirmed) {
				return;
			}

			// ask for element name (when updating element in project)
			final InputDialog updateDialog = new InputDialog(Display.getCurrent().getActiveShell(), shellTitle,
					"Enter the name with which the element should be updated in the project:\n(Current name in project: "
							+ currentNameInProject + ")\n ",
					currentNameInProject, new UpdatedNameValidator());
			if (updateDialog.open() != OK) {
				return;
			}
			String updateName = updateDialog.getValue();

			// execute "updating" action
			ICommandStackService.getInstance().runAsCommand(selectedElement, () -> {
				updateElemFromLib(selectedElement, correctLibrary, correctLibrarySpec, updateName);
			});
		}

		/** Validator for the name of the updated element. */
		private class UpdatedNameValidator implements IInputValidator {

			/** {@inheritDoc} */
			@Override
			public String isValid(String newText) {
				if (newText.equals("")) {
					return "A name needs to be specified.";
				}
				return null;
			}
		}

	}
}
