/*-------------------------------------------------------------------------+
| Copyright 2021 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.ext.reuse.ui.dialog;

import static org.eclipse.jface.dialogs.IDialogConstants.CANCEL_ID;
import static org.eclipse.jface.dialogs.IDialogConstants.CANCEL_LABEL;
import static org.eclipse.jface.dialogs.IDialogConstants.OK_ID;
import static org.eclipse.jface.dialogs.IDialogConstants.OK_LABEL;
import static org.eclipse.swt.SWT.CENTER;
import static org.eclipse.swt.SWT.LEFT;
import static org.eclipse.swt.SWT.NONE;
import static org.eclipse.swt.SWT.READ_ONLY;
import static org.eclipse.swt.SWT.TITLE;
import static org.fortiss.tooling.ext.reuse.utils.ReuseLibraryUtils.getLibraryIDName;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.fortiss.tooling.ext.reuse.model.ReuseLibrary;

/**
 * {@link Dialog} class for adding an element to a {@link ReuseLibrary}.
 *
 * @author bergemann
 */
public class AddElementToLibDialog extends Dialog {

	private static final int CREATE_ID = 100;

	/** The title of the dialog. */
	private String title;

	/** The combo format. */
	private Combo comboFormat;

	/** The selected {@link ReuseLibrary}. */
	private ReuseLibrary library;

	/** The "go ahead" flag, which is true if the user wants to continue. */
	private boolean goAhead = false;

	/** The "create new library" flag. */
	private boolean createNew = false;

	/** The collection of possible {@link ReuseLibrary}s. */
	private List<ReuseLibrary> libraries = new ArrayList<ReuseLibrary>();

	/**
	 * Creates the dialog.
	 *
	 * @param parentShell The parent shell
	 * @param dialogTitle The title of the dialog shell
	 */
	public AddElementToLibDialog(Shell parentShell, String dialogTitle) {
		super(parentShell);
		this.title = dialogTitle;
		setShellStyle(TITLE);
	}

	@Override
	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		if (title != null) {
			shell.setText(title);
		}
	}

	/** {@inheritDoc} */
	@Override
	protected void buttonPressed(int buttonId) {
		super.buttonPressed(buttonId);
		if (buttonId == CREATE_ID) {
			createPressed();
		}
	}

	/** {@inheritDoc} */
	@Override
	protected void okPressed() {
		library = libraries.get(comboFormat.getSelectionIndex());
		goAhead = true;
		super.okPressed();
	}

	protected void createPressed() {
		library = null;
		createNew = true;
		goAhead = true;
		super.okPressed();
	}

	/** {@inheritDoc} */
	@Override
	protected Control createDialogArea(Composite parent) {
		parent.setToolTipText("");
		final Composite container = (Composite) super.createDialogArea(parent);

		final Label initText = new Label(container, NONE);
		initText.setLayoutData(new GridData(LEFT, CENTER, false, false, 1, 1));
		initText.setText("Select the library in which the element should be saved:");

		final Composite container2 = new Composite(container, NONE);
		container2.setLayout(new GridLayout(2, false));

		final Label selectionText = new Label(container2, NONE);
		selectionText.setLayoutData(new GridData(LEFT, CENTER, false, false, 1, 1));
		selectionText.setText("Library (name and UUID): ");

		comboFormat = new Combo(container2, NONE | READ_ONLY);
		comboFormat.setItems(getLibraryIDNames(libraries).toArray(new String[0]));
		final GridData gd_combo = new GridData(LEFT, CENTER, true, false, 1, 1);
		gd_combo.widthHint = 315;
		comboFormat.setLayoutData(gd_combo);
		comboFormat.select(0);

		final Label creationText = new Label(container, NONE);
		creationText.setLayoutData(new GridData(LEFT, CENTER, false, false, 1, 1));
		creationText.setText("Or create a new library...");

		return container;
	}

	/** {@inheritDoc} */
	@Override
	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, CREATE_ID, "Create new library", false);
		createButton(parent, OK_ID, OK_LABEL, true);
		createButton(parent, CANCEL_ID, CANCEL_LABEL, false);
	}

	/** {@inheritDoc} */
	@Override
	protected Point getInitialSize() {
		return new Point(520, 190);
	}

	/**
	 * Returns a list of display names with the correct format (name and UUID) of
	 * given {@link ReuseLibrary}s.
	 */
	protected List<String> getLibraryIDNames(List<ReuseLibrary> libraries) {
		List<String> names = new ArrayList<String>();
		for (ReuseLibrary library : libraries) {
			names.add(getLibraryIDName(library));
		}
		return names;
	}

	/**
	 * Returns the state of the goAhead flag.
	 *
	 * @return boolean flag state
	 */
	public boolean goAhead() {
		return goAhead;
	}

	/**
	 * Returns the {@link ReuseLibrary} that was (maybe) selected during the dialog.
	 *
	 * @return The requested reuse library
	 */
	public ReuseLibrary getLibrary() {
		return library;
	}

	/**
	 * Returns the state of the createNew flag.
	 *
	 * @return boolean flag state
	 */
	public boolean createNew() {
		return createNew;
	}

	/**
	 * Sets the list of all current {@link ReuseLibrary} that can be possibly used
	 * during this dialog.
	 *
	 * @param possibleLibraries A list of all current {@link ReuseLibrary}s
	 */
	public void setLibraryList(List<ReuseLibrary> possibleLibraries) {
		libraries = possibleLibraries;
	}
}
